@@ -1,7 +1,7 @@
 /*
  * Claws Mail templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2013 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 2001-2022 The Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
- * 
  */
 
+#include "config.h"
 #include "defs.h"
 
 #include <glib.h>
@@ -41,7 +41,6 @@
 #include "quote_fmt.h"
 #include "prefs_common.h"
 #include "manual.h"
-#include "gtkutils.h"
 #include "menu.h"
 #include "account.h"
 
@@ -62,6 +61,7 @@ static struct Templates {
 	GtkWidget *entry_to;
 	GtkWidget *entry_cc;	
 	GtkWidget *entry_bcc;
+	GtkWidget *entry_replyto;
 	GtkWidget *text_value;
 } templates;
 
@@ -82,6 +82,7 @@ static struct
 	{"To",		&templates.entry_to,		TRUE, 	NULL},
 	{"Cc",		&templates.entry_cc,		TRUE, 	NULL},
 	{"Bcc",		&templates.entry_bcc,		TRUE, 	NULL},
+	{"Reply-To",	&templates.entry_replyto,	TRUE, 	NULL},
 	{"Subject",	&templates.entry_subject,	FALSE,	NULL},
 	{NULL,		NULL,				FALSE,	NULL}
 };
@@ -146,8 +147,8 @@ static void prefs_template_size_allocate_cb(GtkWidget *widget,
 {
 	cm_return_if_fail(allocation != NULL);
 
-	prefs_common.templateswin_width = allocation->width;
-	prefs_common.templateswin_height = allocation->height;
+	gtk_window_get_size(GTK_WINDOW(widget),
+		&prefs_common.templateswin_width, &prefs_common.templateswin_height);
 }
 
 static void prefs_template_window_create(void)
@@ -158,7 +159,7 @@ static void prefs_template_window_create(void)
 	GtkWidget *scrolled_window;
 	GtkWidget   *vpaned;
 	GtkWidget     *vbox1;
-	GtkWidget       *table; /* including : entry_[name|from|to|cc|bcc|subject] */
+	GtkWidget       *table; /* including : entry_[name|from|to|cc|bcc|replyto|subject] */
 	GtkWidget       *scroll2;
 	GtkWidget         *text_value;
 	GtkWidget     *vbox2;
@@ -192,8 +193,9 @@ static void prefs_template_window_create(void)
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_template");
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
+	gtk_window_set_type_hint(GTK_WINDOW(window), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-	vbox = gtk_vbox_new(FALSE, 8);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
@@ -202,28 +204,27 @@ static void prefs_template_window_create(void)
 	gtk_widget_show(scrolled_window);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                         GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(vbox), scrolled_window);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
 
 	/* vpaned to separate template settings from templates list */
-	vpaned = gtk_vpaned_new();
+	vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vpaned);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
- 					      vpaned);
+	gtk_container_add(GTK_CONTAINER(scrolled_window), vpaned);
 	gtk_viewport_set_shadow_type (GTK_VIEWPORT(
 			gtk_bin_get_child(GTK_BIN(scrolled_window))), GTK_SHADOW_NONE);
 
 	/* vbox to handle template name and content */
-	vbox1 = gtk_vbox_new(FALSE, 6);
+	vbox1 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show(vbox1);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);
 	gtk_paned_pack1(GTK_PANED(vpaned), vbox1, FALSE, FALSE);
 
-	table = gtk_table_new(5, 2, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW_2);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+	table = gtk_grid_new();
 	gtk_widget_show(table);
-	gtk_box_pack_start (GTK_BOX (vbox1), table, FALSE, FALSE, 0);
+	gtk_grid_set_row_spacing(GTK_GRID(table), VSPACING_NARROW_2);
+	gtk_grid_set_column_spacing(GTK_GRID(table), 4);
 
+	gtk_box_pack_start (GTK_BOX (vbox1), table, FALSE, FALSE, 0);
 
 	for (i=0; widgets_table[i].label; i++) {
 
@@ -231,20 +232,18 @@ static void prefs_template_window_create(void)
 
 		label = gtk_label_new( (i != 0) ?
 			prefs_common_translated_header_name(widgets_table[i].label) :
-			widgets_table[i].label);
+			_(widgets_table[i].label));
 		gtk_widget_show(label);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, (i + 1),
-				(GtkAttachOptions) (GTK_FILL),
-				(GtkAttachOptions) 0, 0, 0);
-		gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-
+		gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+		gtk_grid_attach(GTK_GRID(table), label, 0, i, 1, 1);
+	
 		*(widgets_table[i].entry) = gtk_entry_new();
 		gtk_widget_show(*(widgets_table[i].entry));
-		gtk_table_attach(GTK_TABLE(table), *(widgets_table[i].entry), 1, 2, i, (i + 1),
-				(GtkAttachOptions) (GTK_EXPAND|GTK_SHRINK|GTK_FILL),
-				(GtkAttachOptions) 0, 0, 0);
+		gtk_grid_attach(GTK_GRID(table), *(widgets_table[i].entry), 1, i, 1, 1);
+		gtk_widget_set_hexpand(*(widgets_table[i].entry), TRUE);
+		gtk_widget_set_halign(*(widgets_table[i].entry), GTK_ALIGN_FILL);
 		CLAWS_SET_TIP(*(widgets_table[i].entry),
-				widgets_table[i].tooltips);
+				_(widgets_table[i].tooltips));
 	}
 
 	/* template content */
@@ -264,41 +263,40 @@ static void prefs_template_window_create(void)
 		font_desc = pango_font_description_from_string
 						(prefs_common.textfont);
 		if (font_desc) {
-			gtk_widget_modify_font(text_value, font_desc);
+			gtk_widget_override_font(text_value, font_desc);
 			pango_font_description_free(font_desc);
 		}
 	}
 	gtk_widget_show(text_value);
 #ifndef GENERIC_UMPC
-	gtk_widget_set_size_request(text_value, -1, 120);
+	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scroll2), 120);
 #else
-	gtk_widget_set_size_request(text_value, -1, 60);
+	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scroll2), 60);
 #endif
 	gtk_container_add(GTK_CONTAINER(scroll2), text_value);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_value), TRUE);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_value), GTK_WRAP_WORD);
 
 	/* vbox for buttons and templates list */
-	vbox2 = gtk_vbox_new(FALSE, 6);
+	vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show(vbox2);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 8);
 	gtk_paned_pack2(GTK_PANED(vpaned), vbox2, TRUE, FALSE);
 
 	/* register | substitute | delete */
-	hbox2 = gtk_hbox_new(FALSE, 4);
+	hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 	gtk_widget_show(hbox2);
 	gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, FALSE, 0);
 
-	arrow1 = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
+	arrow1 = gtk_image_new_from_icon_name("pan-down-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(arrow1);
 	gtk_box_pack_start(GTK_BOX(hbox2), arrow1, FALSE, FALSE, 0);
-	gtk_widget_set_size_request(arrow1, -1, 16);
 
-	hbox3 = gtk_hbox_new(TRUE, 4);
+	hbox3 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 	gtk_widget_show(hbox3);
 	gtk_box_pack_start(GTK_BOX(hbox2), hbox3, FALSE, FALSE, 0);
 
-	reg_btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
+	reg_btn = gtkut_stock_button("list-add", _("_Add"));
 	gtk_widget_show(reg_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), reg_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
@@ -306,7 +304,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(reg_btn,
 			_("Append the new template above to the list"));
 
-	subst_btn = gtkut_get_replace_btn(_("_Replace"));
+	subst_btn = gtkut_stock_button("edit-redo", _("_Replace"));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), subst_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(subst_btn), "clicked",
@@ -315,9 +313,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(subst_btn,
 			_("Replace the selected template in list with the template above"));
 
-	del_btn = gtk_button_new_with_mnemonic (_("Re_move"));
-	gtk_button_set_image(GTK_BUTTON(del_btn),
-			gtk_image_new_from_stock(GTK_STOCK_REMOVE,GTK_ICON_SIZE_BUTTON));
+	del_btn = gtkut_stock_button("edit-delete", _("D_elete"));
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), del_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
@@ -325,9 +321,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(del_btn,
 			_("Delete the selected template from the list"));
 
-	clear_btn = gtk_button_new_with_mnemonic (_("C_lear"));
-	gtk_button_set_image(GTK_BUTTON(clear_btn),
-			gtk_image_new_from_stock(GTK_STOCK_CLEAR,GTK_ICON_SIZE_BUTTON));
+	clear_btn = gtkut_stock_button("edit-clear", _("C_lear"));
 	gtk_widget_show (clear_btn);
 	gtk_box_pack_start (GTK_BOX (hbox3), clear_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (clear_btn), "clicked",
@@ -335,7 +329,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(clear_btn,
 			_("Clear all the input fields in the dialog"));
 
-	desc_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
+	desc_btn = gtkut_stock_button("dialog-information", _("_Information"));
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(desc_btn), "clicked",
@@ -344,7 +338,7 @@ static void prefs_template_window_create(void)
 			_("Show information on configuring templates"));
 
 	/* templates list */
-	hbox4 = gtk_hbox_new(FALSE, 8);
+	hbox4 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
 	gtk_widget_show(hbox4);
 	gtk_box_pack_start(GTK_BOX(vbox2), hbox4, TRUE, TRUE, 0);
 
@@ -355,11 +349,11 @@ static void prefs_template_window_create(void)
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
 				       
-	vbox3 = gtk_vbox_new(FALSE, 8);
+	vbox3 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
 	gtk_widget_show(vbox3);
 	gtk_box_pack_start(GTK_BOX(hbox4), vbox3, FALSE, FALSE, 0);
 
-	top_btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
+	top_btn = gtkut_stock_button("go-top", _("_Top"));
 	gtk_widget_show(top_btn);
 	gtk_box_pack_start(GTK_BOX(vbox3), top_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(top_btn), "clicked",
@@ -367,9 +361,9 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(top_btn,
 			_("Move the selected template to the top"));
 
-	PACK_VSPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
+	PACK_SPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
 
-	up_btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+	up_btn = gtkut_stock_button("go-up", _("_Up"));
 	gtk_widget_show(up_btn);
 	gtk_box_pack_start (GTK_BOX(vbox3), up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(up_btn), "clicked",
@@ -377,7 +371,7 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(up_btn,
 			_("Move the selected template up"));
 
-	down_btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+	down_btn = gtkut_stock_button("go-down", _("_Down"));
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start(GTK_BOX (vbox3), down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (down_btn), "clicked",
@@ -385,9 +379,9 @@ static void prefs_template_window_create(void)
 	CLAWS_SET_TIP(down_btn,
 			_("Move the selected template down"));
 
-	PACK_VSPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
+	PACK_SPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
 
-	bottom_btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_BOTTOM);
+	bottom_btn = gtkut_stock_button("go-bottom", _("_Bottom"));
 	gtk_widget_show(bottom_btn);
 	gtk_box_pack_start(GTK_BOX(vbox3), bottom_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(bottom_btn), "clicked",
@@ -402,9 +396,9 @@ static void prefs_template_window_create(void)
 
 	/* help | cancel | ok */
 	gtkut_stock_button_set_create_with_help(&confirm_area, &help_btn,
-			&cancel_btn, GTK_STOCK_CANCEL,
-			&ok_btn, GTK_STOCK_OK,
-			NULL, NULL);
+			&cancel_btn, NULL, _("_Cancel"),
+			&ok_btn, NULL, _("_OK"),
+			NULL, NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -433,7 +427,8 @@ static void prefs_template_window_create(void)
 
 	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
 				      GDK_HINT_MIN_SIZE);
-	gtk_widget_set_size_request(window, prefs_common.templateswin_width,
+	gtk_window_set_default_size(GTK_WINDOW(window),
+				    prefs_common.templateswin_width,
 				    prefs_common.templateswin_height);
 
 	templates.window = window;
@@ -451,6 +446,7 @@ static void prefs_template_reset_dialog(void)
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc), "");			
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_replyto), "");			
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject), "");
 	
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
@@ -568,8 +564,8 @@ static void prefs_template_ok_cb(gpointer action, gpointer data)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("+_Continue editing"), 
-				 NULL) != G_ALERTDEFAULT) {
+				 "window-close", _("_Close"), NULL, _("_Continue editing"),
+				 NULL, NULL, ALERTFOCUS_SECOND) != G_ALERTDEFAULT) {
 		return;
 	} 
 
@@ -593,13 +589,13 @@ static void prefs_template_cancel_cb(gpointer action, gpointer data)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("+_Continue editing"),
-				 NULL) != G_ALERTDEFAULT) {
+				 "window-close", _("_Close"), NULL, _("_Continue editing"),
+				 NULL, NULL, ALERTFOCUS_SECOND) != G_ALERTDEFAULT) {
 		return;
 	} else if (modified_list && alertpanel(_("Templates list not saved"),
 				 _("The templates list has been modified. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("+_Continue editing"), 
-				 NULL) != G_ALERTDEFAULT) {
+				 "window-close", _("_Close"), NULL, _("_Continue editing"),
+				 NULL, NULL, ALERTFOCUS_SECOND) != G_ALERTDEFAULT) {
 		return;
 	}
 
@@ -657,6 +653,9 @@ static GSList *prefs_template_get_list(void)
 			ntmpl->bcc     = tmpl->bcc && *(tmpl->bcc)
 					 ? g_strdup(tmpl->bcc)
 					 : NULL;	
+			ntmpl->replyto = tmpl->replyto && *(tmpl->replyto)
+					 ? g_strdup(tmpl->replyto)
+					 : NULL;	
 			ntmpl->value   = tmpl->value && *(tmpl->value)
 			                 ? g_strdup(tmpl->value)
 					 : NULL;
@@ -708,19 +707,11 @@ gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean esca
 		if (!parsed_buf) {
 			if (line)
 				*line = quote_fmt_get_line();
+			quote_fmtlex_destroy();
 			return FALSE;
 		}
-		if (email) {
-			const gchar *start = strrchr(parsed_buf, '<');
-			const gchar *end = strrchr(parsed_buf, '>');
-			const gchar *at = strrchr(parsed_buf, '@');
-			const gchar *space = strrchr(parsed_buf, ' ');
-			if (!at)
-				result = FALSE;
-			if (at && space && (!start || !end || end < start || start < space))
-				result = FALSE;
-		}
 		quote_fmt_reset_vartable();
+		quote_fmtlex_destroy();
 	}
 	return result;
 }
@@ -734,7 +725,8 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	gchar *from;
 	gchar *to;
 	gchar *cc;
-	gchar *bcc;	
+	gchar *bcc;
+	gchar *replyto;
 	gchar *value;
 	GtkTextBuffer *buffer;
 	GtkTextIter start, end;
@@ -770,6 +762,8 @@ static gboolean prefs_template_list_view_set_row(gint row)
 				    0, -1);
 	bcc = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_bcc),
 				    0, -1);
+	replyto = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_replyto),
+				    0, -1);
 	subject = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_subject),
 					 0, -1);
 
@@ -789,6 +783,10 @@ static gboolean prefs_template_list_view_set_row(gint row)
 		g_free(bcc);
 		bcc = NULL;
 	}
+	if (replyto && *replyto == '\0') {
+		g_free(replyto);
+		replyto = NULL;
+	}
 	if (subject && *subject == '\0') {
 		g_free(subject);
 		subject = NULL;
@@ -818,6 +816,12 @@ static gboolean prefs_template_list_view_set_row(gint row)
 		g_free(value);
 		return FALSE;
 	}
+	if (!prefs_template_string_is_valid(replyto, NULL, TRUE, TRUE)) {
+		alertpanel_error(_("The \"Reply-To\" field of the template contains an invalid email address."));	
+		g_free(replyto);
+		g_free(value);
+		return FALSE;
+	}
 	if (!prefs_template_string_is_valid(subject, NULL, TRUE, FALSE)) {
 		alertpanel_error(_("The \"Subject\" field of the template is invalid."));	
 		g_free(subject);
@@ -833,6 +837,7 @@ static gboolean prefs_template_list_view_set_row(gint row)
 	tmpl->to = to;
 	tmpl->cc = cc;
 	tmpl->bcc = bcc;	
+	tmpl->replyto = replyto;	
 	tmpl->value = value;
 
 	prefs_template_list_view_insert_template(templates.list_view,
@@ -891,8 +896,8 @@ static void prefs_template_delete_cb(gpointer action, gpointer data)
 
 	if (alertpanel(_("Delete template"),
 		       _("Do you really want to delete this template?"),
-		       GTK_STOCK_CANCEL, GTK_STOCK_DELETE,
-		       NULL) != G_ALERTALTERNATE)
+		       NULL, _("_Cancel"), "edit-delete", _("D_elete"), NULL, NULL,
+		       ALERTFOCUS_FIRST) != G_ALERTALTERNATE)
 		return;
 
 	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
@@ -904,7 +909,8 @@ static void prefs_template_delete_all_cb(gpointer action, gpointer data)
 {
 	if (alertpanel(_("Delete all templates"),
 			  _("Do you really want to delete all the templates?"),
-			  GTK_STOCK_CANCEL, "+"GTK_STOCK_DELETE, NULL) == G_ALERTDEFAULT)
+			  NULL, _("_Cancel"), "edit-delete", _("D_elete"), NULL, NULL,
+			  ALERTFOCUS_SECOND) == G_ALERTDEFAULT)
 	   return;
 
 	prefs_template_clear_list();
@@ -1095,7 +1101,7 @@ static GtkWidget *prefs_template_popup_menu = NULL;
 
 static GtkActionEntry prefs_template_popup_entries[] =
 {
-	{"PrefsTemplatePopup",			NULL, "PrefsTemplatePopup" },
+	{"PrefsTemplatePopup",			NULL, "PrefsTemplatePopup", NULL, NULL, NULL },
 	{"PrefsTemplatePopup/Delete",		NULL, N_("_Delete"), NULL, NULL, G_CALLBACK(prefs_template_delete_cb) },
 	{"PrefsTemplatePopup/DeleteAll",	NULL, N_("Delete _all"), NULL, NULL, G_CALLBACK(prefs_template_delete_all_cb) },
 	{"PrefsTemplatePopup/Duplicate",	NULL, N_("D_uplicate"), NULL, NULL, G_CALLBACK(prefs_template_duplicate_cb) },
@@ -1161,9 +1167,7 @@ static gint prefs_template_list_btn_pressed(GtkWidget *widget, GdkEventButton *e
 			   non_empty = gtk_tree_model_iter_next(model, &iter);
 			cm_menu_set_sensitive("PrefsTemplatePopup/DeleteAll", non_empty);
 
-		   gtk_menu_popup(GTK_MENU(prefs_template_popup_menu), 
-					  NULL, NULL, NULL, NULL, 
-					  event->button, event->time);
+		   gtk_menu_popup_at_pointer(GTK_MENU(prefs_template_popup_menu), NULL);
 	   }
    }
    return FALSE;
@@ -1252,6 +1256,7 @@ static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path)
 	tmpl_def.to = "";
 	tmpl_def.cc = "";
 	tmpl_def.bcc = "";	
+	tmpl_def.replyto = "";	
 	tmpl_def.value = "";
 
 	gtk_tree_model_get(model, &titer, TEMPL_DATA, &tmpl, -1);
@@ -1267,6 +1272,8 @@ static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path)
 			   tmpl->cc ? tmpl->cc : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc),
 			   tmpl->bcc ? tmpl->bcc : "");			
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_replyto),
+			   tmpl->replyto ? tmpl->replyto : "");			
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject),
 			   tmpl->subject ? tmpl->subject : "");
 	
