@@ -1,6 +1,6 @@
 /*
- * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
+ * Claws Mail -- a GTK based, lightweight, and fast e-mail client
+ * Copyright (C) 1999-2023 the Claws Mail team and Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,15 +30,14 @@
 #include <time.h>
 #include <sys/stat.h>
 
-#ifdef G_OS_WIN32
-#  include <w32lib.h>
-#endif
-
 #include "procheader.h"
 #include "procmsg.h"
 #include "codeconv.h"
 #include "prefs_common.h"
+#include "hooks.h"
 #include "utils.h"
+#include "defs.h"
+#include "file-utils.h"
 
 #define BUFFSIZE	8192
 
@@ -47,15 +46,15 @@ static gchar monthstr[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
 typedef char *(*getlinefunc) (char *, size_t, void *);
 typedef int (*peekcharfunc) (void *);
 typedef int (*getcharfunc) (void *);
-typedef gint (*get_one_field_func) (gchar *, size_t, void *, HeaderEntry[]);
+typedef gint (*get_one_field_func) (gchar **, void *, HeaderEntry[]);
 
-static gint string_get_one_field(gchar *buf, size_t len, char **str,
+static gint string_get_one_field(gchar **buf, char **str,
 				 HeaderEntry hentry[]);
 
 static char *string_getline(char *buf, size_t len, char **str);
 static int string_peekchar(char **str);
 static int file_peekchar(FILE *fp);
-static gint generic_get_one_field(gchar *buf, size_t len, void *data,
+static gint generic_get_one_field(gchar **bufptr, void *data,
 				  HeaderEntry hentry[],
 				  getlinefunc getline, 
 				  peekcharfunc peekchar,
@@ -64,23 +63,41 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			     gboolean full, gboolean decrypted);
 
 
-gint procheader_get_one_field(gchar *buf, size_t len, FILE *fp,
+gint procheader_get_one_field(gchar **buf, FILE *fp,
 			      HeaderEntry hentry[])
 {
-	return generic_get_one_field(buf, len, fp, hentry,
+	return generic_get_one_field(buf, fp, hentry,
 				     (getlinefunc)fgets_crlf, (peekcharfunc)file_peekchar,
 				     TRUE);
 }
 
-static gint string_get_one_field(gchar *buf, size_t len, char **str,
+static gint string_get_one_field(gchar **buf, char **str,
 				 HeaderEntry hentry[])
 {
-	return generic_get_one_field(buf, len, str, hentry,
+	return generic_get_one_field(buf, str, hentry,
 				     (getlinefunc)string_getline,
 				     (peekcharfunc)string_peekchar,
 				     TRUE);
 }
 
+gboolean procheader_skip_headers(FILE *fp)
+{
+	gchar *buf = g_malloc(BUFFSIZE);
+	do {
+		if (fgets_crlf(buf, BUFFSIZE - 1, fp) == NULL) {
+			g_free(buf);
+			return FALSE;
+		}
+		if (buf[0] == '\r' || buf[0] == '\n') {
+			break;
+		}
+	} while (TRUE);
+	g_free(buf);
+
+	return TRUE;
+}
+
+
 static char *string_getline(char *buf, size_t len, char **str)
 {
 	gboolean is_cr = FALSE;
@@ -117,23 +134,44 @@ static int file_peekchar(FILE *fp)
 	return ungetc(getc(fp), fp);
 }
 
-static gint generic_get_one_field(gchar *buf, size_t len, void *data,
+static gint generic_get_one_field(gchar **bufptr, void *data,
 			  HeaderEntry *hentry,
 			  getlinefunc getline, peekcharfunc peekchar,
 			  gboolean unfold)
 {
+	/* returns -1 in case of failure of any kind, whatever it's a parsing error
+	   or an allocation error. if returns -1, *bufptr is always NULL, and vice-versa,
+	   and if returning 0 (OK), *bufptr is always non-NULL, so callers just have to
+	   test the return value
+	*/
 	gint nexthead;
 	gint hnum = 0;
 	HeaderEntry *hp = NULL;
+	size_t len;
+	gchar *buf;
+
+	cm_return_val_if_fail(bufptr != NULL, -1);
+
+	len = BUFFSIZE;
+	buf = g_malloc(len);
 
 	if (hentry != NULL) {
 		/* skip non-required headers */
+		/* and get hentry header line */
 		do {
 			do {
-				if (getline(buf, len, data) == NULL)
+				if (getline(buf, len, data) == NULL) {
+					debug_print("generic_get_one_field: getline\n");
+					g_free(buf);
+					*bufptr = NULL;
 					return -1;
-				if (buf[0] == '\r' || buf[0] == '\n')
+				}
+				if (buf[0] == '\r' || buf[0] == '\n') {
+					debug_print("generic_get_one_field: empty line\n");
+					g_free(buf);
+					*bufptr = NULL;
 					return -1;
+				}
 			} while (buf[0] == ' ' || buf[0] == '\t');
 
 			for (hp = hentry, hnum = 0; hp->name != NULL;
@@ -144,8 +182,27 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 			}
 		} while (hp->name == NULL);
 	} else {
-		if (getline(buf, len, data) == NULL) return -1;
-		if (buf[0] == '\r' || buf[0] == '\n') return -1;
+		/* read first line */
+		if (getline(buf, len, data) == NULL) {
+			debug_print("generic_get_one_field: getline\n");
+			g_free(buf);
+			*bufptr = NULL;
+			return -1;
+		}
+		if (buf[0] == '\r' || buf[0] == '\n') {
+			debug_print("generic_get_one_field: empty line\n");
+			g_free(buf);
+			*bufptr = NULL;
+			return -1;
+		}
+	}
+	/* reduce initial buffer to its useful part */
+	len = strlen(buf)+1;
+	buf = g_realloc(buf, len);
+	if (buf == NULL) {
+		debug_print("generic_get_one_field: reallocation error\n");
+		*bufptr = NULL;
+		return -1;
 	}
 
 	/* unfold line */
@@ -154,6 +211,9 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 		/* ([*WSP CRLF] 1*WSP) */
 		if (nexthead == ' ' || nexthead == '\t') {
 			size_t buflen;
+			gchar *tmpbuf;
+			size_t tmplen;
+
 			gboolean skiptab = (nexthead == '\t');
 			/* trim previous trailing \n if requesting one header or
 			 * unfolding was requested */
@@ -162,15 +222,29 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 
 			buflen = strlen(buf);
 			
-			/* concatenate next line */
-			if ((len - buflen) > 2) {
-				if (getline(buf + buflen, len - buflen, data) == NULL)
-					break;
-				if (skiptab) { /* replace tab with space */
-					*(buf + buflen) = ' ';
-				}
-			} else
+			/* read next line */
+			tmpbuf = g_malloc(BUFFSIZE);
+
+			if (getline(tmpbuf, BUFFSIZE, data) == NULL) {
+				g_free(tmpbuf);
 				break;
+			}
+			tmplen = strlen(tmpbuf)+1;
+
+			/* extend initial buffer and concatenate next line */
+			len += tmplen;
+			buf = g_realloc(buf, len);
+			if (buf == NULL) {
+				debug_print("generic_get_one_field: reallocation error\n");
+				g_free(buf);
+				*bufptr = NULL;
+				return -1;
+			}
+			memcpy(buf+buflen, tmpbuf, tmplen);
+			g_free(tmpbuf);
+			if (skiptab) { /* replace tab with space */
+				*(buf + buflen) = ' ';
+			}
 		} else {
 			/* remove trailing new line */
 			strretchomp(buf);
@@ -178,20 +252,22 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 		}
 	}
 
+	*bufptr = buf;
+
 	return hnum;
 }
 
-gint procheader_get_one_field_asis(gchar *buf, size_t len, FILE *fp)
+gint procheader_get_one_field_asis(gchar **buf, FILE *fp)
 {
-	return generic_get_one_field(buf, len, fp, NULL,
+	return generic_get_one_field(buf, fp, NULL,
 				     (getlinefunc)fgets_crlf, 
 				     (peekcharfunc)file_peekchar,
 				     FALSE);
 }
 
-GPtrArray *procheader_get_header_array_asis(FILE *fp)
+GPtrArray *procheader_get_header_array(FILE *fp)
 {
-	gchar buf[BUFFSIZE];
+	gchar *buf = NULL;
 	GPtrArray *headers;
 	Header *header;
 
@@ -199,9 +275,11 @@ GPtrArray *procheader_get_header_array_asis(FILE *fp)
 
 	headers = g_ptr_array_new();
 
-	while (procheader_get_one_field_asis(buf, sizeof(buf), fp) != -1) {
+	while (procheader_get_one_field(&buf, fp, NULL) != -1) {
 		if ((header = procheader_parse_header(buf)) != NULL)
 			g_ptr_array_add(headers, header);
+		g_free(buf);
+		buf = NULL;
 	}
 
 	return headers;
@@ -212,6 +290,8 @@ void procheader_header_array_destroy(GPtrArray *harray)
 	gint i;
 	Header *header;
 
+	cm_return_if_fail(harray != NULL);
+
 	for (i = 0; i < harray->len; i++) {
 		header = g_ptr_array_index(harray, i);
 		procheader_header_free(header);
@@ -288,6 +368,8 @@ Header * procheader_parse_header(gchar * buf)
 	Header * header;
 	gboolean addr_field = FALSE;
 
+	cm_return_val_if_fail(buf != NULL, NULL);
+
 	if ((*buf == ':') || (*buf == ' '))
 		return NULL;
 
@@ -307,15 +389,14 @@ Header * procheader_parse_header(gchar * buf)
 
 void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 {
-	gchar buf[BUFFSIZE];
+	gchar *buf = NULL;
 	HeaderEntry *hp;
 	gint hnum;
 	gchar *p;
 
 	if (hentry == NULL) return;
 
-	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
-	       != -1) {
+	while ((hnum = procheader_get_one_field(&buf, fp, hentry)) != -1) {
 		hp = hentry + hnum;
 
 		p = buf + strlen(hp->name);
@@ -329,36 +410,76 @@ void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 			hp->body = g_strconcat(tp, ", ", p, NULL);
 			g_free(tp);
 		}
+		g_free(buf);
+		buf = NULL;
 	}
 }
 
 MsgInfo *procheader_parse_file(const gchar *file, MsgFlags flags,
 			       gboolean full, gboolean decrypted)
 {
-	struct stat s;
+#ifdef G_OS_WIN32
+	GFile *f;
+	GFileInfo *fi;
+	GTimeVal tv;
+	GError *error = NULL;
+#else
+	GStatBuf s;
+#endif
 	FILE *fp;
 	MsgInfo *msginfo;
 
+#ifdef G_OS_WIN32
+	f = g_file_new_for_path(file);
+	fi = g_file_query_info(f, "standard::size,standard::type,time::modified",
+			G_FILE_QUERY_INFO_NONE, NULL, &error);
+	if (error != NULL) {
+		g_warning(error->message);
+		g_error_free(error);
+		g_object_unref(f);
+	}
+#else
 	if (g_stat(file, &s) < 0) {
 		FILE_OP_ERROR(file, "stat");
 		return NULL;
 	}
+#endif
+
+#ifdef G_OS_WIN32
+	if (g_file_info_get_file_type(fi) != G_FILE_TYPE_REGULAR) {
+		g_object_unref(fi);
+		g_object_unref(f);
+		return NULL;
+	}
+#else
 	if (!S_ISREG(s.st_mode))
 		return NULL;
+#endif
 
-	if ((fp = g_fopen(file, "rb")) == NULL) {
-		FILE_OP_ERROR(file, "fopen");
+	if ((fp = claws_fopen(file, "rb")) == NULL) {
+		FILE_OP_ERROR(file, "claws_fopen");
 		return NULL;
 	}
 
 	msginfo = procheader_parse_stream(fp, flags, full, decrypted);
-	fclose(fp);
+	claws_fclose(fp);
 
 	if (msginfo) {
+#ifdef G_OS_WIN32
+		msginfo->size = g_file_info_get_size(fi);
+		g_file_info_get_modification_time(fi, &tv);
+		msginfo->mtime = tv.tv_sec;
+#else
 		msginfo->size = s.st_size;
 		msginfo->mtime = s.st_mtime;
+#endif
 	}
 
+#ifdef G_OS_WIN32
+	g_object_unref(fi);
+	g_object_unref(f);
+#endif
+
 	return msginfo;
 }
 
@@ -370,38 +491,39 @@ MsgInfo *procheader_parse_str(const gchar *str, MsgFlags flags, gboolean full,
 
 enum
 {
-	H_DATE		= 0,
-	H_FROM		= 1,
-	H_TO		= 2,
-	H_CC		= 3,
-	H_NEWSGROUPS	= 4,
-	H_SUBJECT	= 5,
-	H_MSG_ID	= 6,
-	H_REFERENCES	= 7,
-	H_IN_REPLY_TO	= 8,
-	H_CONTENT_TYPE	= 9,
-	H_SEEN		= 10,
-	H_STATUS        = 11,
-	H_X_STATUS      = 12,
-	H_FROM_SPACE	= 13,
-	H_SC_PLANNED_DOWNLOAD = 14,
-	H_SC_MESSAGE_SIZE = 15,
-	H_FACE		= 16,
-	H_X_FACE	= 17,
-	H_DISPOSITION_NOTIFICATION_TO = 18,
-	H_RETURN_RECEIPT_TO = 19,
-	H_SC_PARTIALLY_RETRIEVED = 20,
-	H_SC_ACCOUNT_SERVER = 21,
-	H_SC_ACCOUNT_LOGIN = 22,
- 	H_LIST_POST	   = 23,
-	H_LIST_SUBSCRIBE   = 24,
- 	H_LIST_UNSUBSCRIBE = 25,
- 	H_LIST_HELP        = 26,
- 	H_LIST_ARCHIVE     = 27,
- 	H_LIST_OWNER       = 28,
+	H_DATE = 0,
+	H_FROM,
+	H_TO,
+	H_CC,
+	H_NEWSGROUPS,
+	H_SUBJECT,
+	H_MSG_ID,
+	H_REFERENCES,
+	H_IN_REPLY_TO,
+	H_CONTENT_TYPE,
+	H_SEEN,
+	H_STATUS,
+	H_FROM_SPACE,
+	H_SC_PLANNED_DOWNLOAD,
+	H_SC_MESSAGE_SIZE,
+	H_FACE,
+	H_X_FACE,
+	H_DISPOSITION_NOTIFICATION_TO,
+	H_RETURN_RECEIPT_TO,
+	H_SC_PARTIALLY_RETRIEVED,
+	H_SC_ACCOUNT_SERVER,
+	H_SC_ACCOUNT_LOGIN,
+	H_LIST_POST,
+	H_LIST_SUBSCRIBE,
+	H_LIST_UNSUBSCRIBE,
+	H_LIST_HELP,
+	H_LIST_ARCHIVE,
+	H_LIST_OWNER,
+	H_RESENT_FROM,
 };
 
-static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
+static HeaderEntry hentry_full[] = {
+				   {"Date:",		NULL, FALSE},
 				   {"From:",		NULL, TRUE},
 				   {"To:",		NULL, TRUE},
 				   {"Cc:",		NULL, TRUE},
@@ -413,7 +535,6 @@ static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
 				   {"Content-Type:",	NULL, FALSE},
 				   {"Seen:",		NULL, FALSE},
 				   {"Status:",          NULL, FALSE},
-				   {"X-Status:",        NULL, FALSE},
 				   {"From ",		NULL, FALSE},
 				   {"SC-Marked-For-Download:", NULL, FALSE},
 				   {"SC-Message-Size:", NULL, FALSE},
@@ -430,9 +551,11 @@ static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
 				   {"List-Help:",	NULL, TRUE},
  				   {"List-Archive:",	NULL, TRUE},
  				   {"List-Owner:",	NULL, TRUE},
+ 				   {"Resent-From:",	NULL, TRUE},
 				   {NULL,		NULL, FALSE}};
 
-static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
+static HeaderEntry hentry_short[] = {
+				    {"Date:",		NULL, FALSE},
 				    {"From:",		NULL, TRUE},
 				    {"To:",		NULL, TRUE},
 				    {"Cc:",		NULL, TRUE},
@@ -444,7 +567,6 @@ static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
 				    {"Content-Type:",	NULL, FALSE},
 				    {"Seen:",		NULL, FALSE},
 				    {"Status:",		NULL, FALSE},
-				    {"X-Status:",	NULL, FALSE},
 				    {"From ",		NULL, FALSE},
 				    {"SC-Marked-For-Download:", NULL, FALSE},
 				    {"SC-Message-Size:",NULL, FALSE},
@@ -461,11 +583,33 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full,
 	return parse_stream(fp, FALSE, flags, full, decrypted);
 }
 
+static gboolean avatar_from_some_face(gpointer source, gpointer userdata)
+{
+	AvatarCaptureData *acd = (AvatarCaptureData *)source;
+	
+	if (*(acd->content) == '\0') /* won't be null, but may be empty */
+		return FALSE;
+
+	if (!strcmp(acd->header, hentry_full[H_FACE].name)) {
+		debug_print("avatar_from_some_face: found 'Face' header\n");
+		procmsg_msginfo_add_avatar(acd->msginfo, AVATAR_FACE, acd->content);
+	}
+#if HAVE_LIBCOMPFACE
+	else if (!strcmp(acd->header, hentry_full[H_X_FACE].name)) {
+		debug_print("avatar_from_some_face: found 'X-Face' header\n");
+		procmsg_msginfo_add_avatar(acd->msginfo, AVATAR_XFACE, acd->content);
+	}
+#endif
+	return FALSE;
+}
+
+static gulong avatar_hook_id = HOOK_NONE;
+
 static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			     gboolean full, gboolean decrypted)
 {
 	MsgInfo *msginfo;
-	gchar buf[BUFFSIZE];
+	gchar *buf = NULL;
 	gchar *p, *tmp;
 	gchar *hp;
 	HeaderEntry *hentry;
@@ -479,12 +623,15 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 	hentry = procheader_get_headernames(full);
 
 	if (MSG_IS_QUEUED(flags) || MSG_IS_DRAFT(flags)) {
-		while (get_one_field(buf, sizeof(buf), data, NULL) != -1) {
+		while (get_one_field(&buf, data, NULL) != -1) {
 			if ((!strncmp(buf, "X-Claws-End-Special-Headers: 1",
 				strlen("X-Claws-End-Special-Headers:"))) ||
 			    (!strncmp(buf, "X-Sylpheed-End-Special-Headers: 1",
-				strlen("X-Sylpheed-End-Special-Headers:"))))
+				strlen("X-Sylpheed-End-Special-Headers:")))) {
+				g_free(buf);
+				buf = NULL;
 				break;
+			}
 			/* from other mailers */
 			if (!strncmp(buf, "Date: ", 6)
 			||  !strncmp(buf, "To: ", 4)
@@ -494,8 +641,12 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 					data = orig_data;
 				else 
 					rewind((FILE *)data);
+				g_free(buf);
+				buf = NULL;
 				break;
 			}
+			g_free(buf);
+			buf = NULL;
 		}
 	}
 
@@ -508,8 +659,17 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 	
 	msginfo->inreplyto = NULL;
 
-	while ((hnum = get_one_field(buf, sizeof(buf), data, hentry))
-	       != -1) {
+	if (avatar_hook_id == HOOK_NONE &&
+	    (prefs_common.enable_avatars && (AVATARS_ENABLE_CAPTURE || AVATARS_ENABLE_RENDER))) {
+		avatar_hook_id = hooks_register_hook(AVATAR_HEADER_UPDATE_HOOKLIST,
+						     avatar_from_some_face, NULL);
+	} else if (avatar_hook_id != HOOK_NONE &&
+		   !(prefs_common.enable_avatars && AVATARS_ENABLE_CAPTURE)) {
+		hooks_unregister_hook(AVATAR_HEADER_UPDATE_HOOKLIST, avatar_hook_id);
+		avatar_hook_id = HOOK_NONE;
+	}
+
+	while ((hnum = get_one_field(&buf, data, hentry)) != -1) {
 		hp = buf + strlen(hentry[hnum].name);
 		while (*hp == ' ' || *hp == '\t') hp++;
 
@@ -608,24 +768,6 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			if (!g_ascii_strncasecmp(hp, "multipart/", 10))
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MULTIPART);
 			break;
-#ifdef ALLOW_HEADER_HINT			
-		case H_SEEN:
-			/* mnews Seen header */
-			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
-			break;
-#endif			
-		case H_FACE:
-			if (!msginfo->extradata)
-				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
-			if (msginfo->extradata->face) break;
-			msginfo->extradata->face = g_strdup(hp);
-			break;
-		case H_X_FACE:
-			if (!msginfo->extradata)
-				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
-			if (msginfo->extradata->xface) break;
-			msginfo->extradata->xface = g_strdup(hp);
-			break;
 		case H_DISPOSITION_NOTIFICATION_TO:
 			if (!msginfo->extradata)
 				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
@@ -665,29 +807,6 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			msginfo->planned_download = atoi(hp);
 			break;
 /* end partial download infos */
-#ifdef ALLOW_HEADER_HINT			
-		case H_STATUS:
-			if (strchr(hp, 'R') != NULL)
-				MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_UNREAD);
-			if (strchr(hp, 'O') != NULL)
-				MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_NEW);
-			if (strchr(hp, 'U') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_UNREAD);
-			break;
-		case H_X_STATUS:
-			if (strchr(hp, 'D') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags,
-					      MSG_REALLY_DELETED);
-			if (strchr(hp, 'F') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_MARKED);
-			if (strchr(hp, 'd') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_DELETED);
-			if (strchr(hp, 'r') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
-			if (strchr(hp, 'f') != NULL)
-				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_FORWARDED);
-			break;
-#endif			
 		case H_FROM_SPACE:
 			if (msginfo->fromspace) break;
 			msginfo->fromspace = g_strdup(hp);
@@ -730,10 +849,30 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			if (msginfo->extradata->list_owner) break;
 			msginfo->extradata->list_owner = g_strdup(hp);
 			break;
+		case H_RESENT_FROM:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->resent_from) break;
+			msginfo->extradata->resent_from = g_strdup(hp);
+			break;
 /* end list infos */
 		default:
 			break;
 		}
+		/* to avoid performance penalty hooklist is invoked only for
+		   headers known to be able to generate avatars */
+		if (hnum == H_FROM || hnum == H_X_FACE || hnum == H_FACE) {
+			AvatarCaptureData *acd = g_new0(AvatarCaptureData, 1);
+			/* no extra memory is wasted, hooks are expected to
+			   take care of copying members when needed */
+			acd->msginfo = msginfo;
+			acd->header  = hentry_full[hnum].name;
+			acd->content = hp;
+			hooks_invoke(AVATAR_HEADER_UPDATE_HOOKLIST, (gpointer)acd);
+			g_free(acd);
+		}
+		g_free(buf);
+		buf = NULL;
 	}
 
 	if (!msginfo->inreplyto && msginfo->references)
@@ -773,7 +912,56 @@ gchar *procheader_get_fromname(const gchar *str)
 	return name;
 }
 
-static gint procheader_scan_date_string(const gchar *str,
+static gint procheader_remove_comment_in_date_string(gchar *o_str)
+{
+	gchar str[strlen(o_str)+1];
+	int i, j = 0;
+	int in_comment_nest_level = 0;
+	gboolean flag_escape_backslash = FALSE;
+
+	for (i=0; i < strlen(o_str); i++) {
+		switch (o_str[i]) {
+		case '(':
+			in_comment_nest_level++;
+			if (in_comment_nest_level > 16) {
+				str[j] = '\0';
+				return TRUE;
+			}
+			continue;
+		case '\\':
+			if (in_comment_nest_level > 0) {
+				flag_escape_backslash = TRUE;
+				continue;
+			}
+			break;
+		case ')':
+			if (flag_escape_backslash == TRUE) {
+				flag_escape_backslash = FALSE;
+				continue;
+			}
+			in_comment_nest_level--;
+			if (in_comment_nest_level < 0) {
+				str[j] = '\0';
+				return TRUE;
+			}
+			continue;
+		default:
+			if (in_comment_nest_level > 0) {
+				if (flag_escape_backslash == TRUE)
+					flag_escape_backslash = FALSE;
+				continue;
+			}
+			break;
+		}
+		str[j++] = o_str[i];
+	}
+	str[j] = '\0';
+	strcpy(o_str, str);
+	return TRUE;
+}
+
+
+static gint procheader_scan_date_string(const gchar *o_str,
 					gchar *weekday, gint *day,
 					gchar *month, gint *year,
 					gint *hh, gint *mm, gint *ss,
@@ -781,22 +969,34 @@ static gint procheader_scan_date_string(const gchar *str,
 {
 	gint result;
 	gint month_n;
-	gchar zone1[3];
-	gchar zone2[3];
+	gint secfract;
+	gint zone1 = 0, zone2 = 0;
+	gchar offset_sign, zonestr[7];
+	gchar sep1;
 
-	if (str == NULL)
+	if (o_str == NULL)
 		return -1;
 
-	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d %5s",
+	gchar str[strlen(o_str)+1];
+
+	strcpy(str, o_str);
+	if (strchr(str, '(') != NULL)
+		procheader_remove_comment_in_date_string(str);
+
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d %6s",
 			weekday, day, month, year, hh, mm, ss, zone);
 	if (result == 8) return 0;
 
 	/* RFC2822 */
-	result = sscanf(str, "%3s,%d %9s %d %2d:%2d:%2d %5s",
+	result = sscanf(str, "%3s,%d %9s %d %2d:%2d:%2d %6s",
 			weekday, day, month, year, hh, mm, ss, zone);
 	if (result == 8) return 0;
 
-	result = sscanf(str, "%d %9s %d %2d:%2d:%2d %5s",
+	result = sscanf(str, "%3s %3s %d %2d:%2d:%2d %d %6s",
+			weekday, month, day, hh, mm, ss, year, zone);
+	if (result == 8) return 0;
+
+	result = sscanf(str, "%d %9s %d %2d:%2d:%2d %6s",
 			day, month, year, hh, mm, ss, zone);
 	if (result == 7) return 0;
 
@@ -805,12 +1005,16 @@ static gint procheader_scan_date_string(const gchar *str,
 			weekday, day, month, year, hh, mm, ss);
 	if (result == 7) return 0;
 
+	result = sscanf(str, "%3s %3s %d %2d:%2d:%2d %d",
+			weekday, month, day, hh, mm, ss, year);
+	if (result == 7) return 0;
+
 	result = sscanf(str, "%d %9s %d %2d:%2d:%2d",
 			day, month, year, hh, mm, ss);
 	if (result == 6) return 0;
 
 	*ss = 0;
-	result = sscanf(str, "%10s %d %9s %d %2d:%2d %5s",
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d %6s",
 			weekday, day, month, year, hh, mm, zone);
 	if (result == 7) return 0;
 
@@ -827,23 +1031,46 @@ static gint procheader_scan_date_string(const gchar *str,
 			day, month, year, hh, mm);
 	if (result == 5) return 0;
 
-	/* RFC3339 subset */
 	*weekday = '\0';
-	result = sscanf(str, "%4d-%2d-%2d %2d:%2d:%2d+%2s:%2s",
-			year, &month_n, day, hh, mm, ss, zone1, zone2);
-	if (result == 8) {
-		if (1 <= month_n && month_n <= 12) {
+
+	/* RFC3339 subset, with fraction of second */
+	result = sscanf(str, "%4d-%2d-%2d%c%2d:%2d:%2d.%d%6s",
+			year, &month_n, day, &sep1, hh, mm, ss, &secfract, zonestr);
+	if (result == 9
+			&& (sep1 == 'T' || sep1 == 't' || sep1 == ' ')) {
+		if (month_n >= 1 && month_n <= 12) {
 			strncpy2(month, monthstr+((month_n-1)*3), 4);
-			*zone = '+';
-			strncpy2(zone+1, zone1, 3);
-			strncpy2(zone+3, zone2, 3);
+			if (zonestr[0] == 'z' || zonestr[0] == 'Z') {
+				strcat(zone, "+00:00");
+			} else if (sscanf(zonestr, "%c%2d:%2d",
+						&offset_sign, &zone1, &zone2) == 3) {
+				strcat(zone, zonestr);
+			}
+			return 0;
+		}
+	}
+
+	/* RFC3339 subset, no fraction of second */
+	result = sscanf(str, "%4d-%2d-%2d%c%2d:%2d:%2d%6s",
+			year, &month_n, day, &sep1, hh, mm, ss, zonestr);
+	if (result == 8
+			&& (sep1 == 'T' || sep1 == 't' || sep1 == ' ')) {
+		if (month_n >= 1 && month_n <= 12) {
+			strncpy2(month, monthstr+((month_n-1)*3), 4);
+			if (zonestr[0] == 'z' || zonestr[0] == 'Z') {
+				strcat(zone, "+00:00");
+			} else if (sscanf(zonestr, "%c%2d:%2d",
+						&offset_sign, &zone1, &zone2) == 3) {
+				strcat(zone, zonestr);
+			}
 			return 0;
 		}
 	}
 
-	/* RFC3339 subset */
 	*zone = '\0';
-	*weekday = '\0';
+
+	/* RFC3339 subset, no fraction of second, and no timezone offset */
+	/* This particular "subset" is invalid, RFC requires the offset */
 	result = sscanf(str, "%4d-%2d-%2d %2d:%2d:%2d",
 			year, &month_n, day, hh, mm, ss);
 	if (result == 6) {
@@ -853,6 +1080,17 @@ static gint procheader_scan_date_string(const gchar *str,
 		}
 	}
 
+	/* ISO8601 format with just date (YYYY-MM-DD) */
+	result = sscanf(str, "%4d-%2d-%2d",
+			year, &month_n, day);
+	if (result == 3) {
+		*hh = *mm = *ss = 0;
+		if (1 <= month_n && month_n <= 12) {
+			strncpy2(month, monthstr+((month_n-1)*3), 4);
+			return 0;
+		}
+	}
+
 	return -1;
 }
 
@@ -877,7 +1115,7 @@ gboolean procheader_date_parse_to_tm(const gchar *src, struct tm *t, char *zone)
 
 	if (procheader_scan_date_string(src, weekday, &day, month, &year,
 					&hh, &mm, &ss, zone) < 0) {
-		g_warning("Invalid date: %s\n", src);
+		g_warning("invalid date: %s", src);
 		return FALSE;
 	}
 
@@ -893,7 +1131,7 @@ gboolean procheader_date_parse_to_tm(const gchar *src, struct tm *t, char *zone)
 	if ((p = strstr(monthstr, month)) != NULL)
 		dmonth = (gint)(p - monthstr) / 3 + 1;
 	else {
-		g_warning("Invalid month: %s\n", month);
+		g_warning("invalid month: %s", month);
 		dmonth = G_DATE_BAD_MONTH;
 	}
 
@@ -919,12 +1157,10 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	gchar month[10];
 	gint year;
 	gint hh, mm, ss;
-	gchar zone[6];
+	gchar zone[7];
 	GDateMonth dmonth = G_DATE_BAD_MONTH;
-	struct tm t;
 	gchar *p;
 	time_t timer;
-	time_t tz_offset;
 
 	if (procheader_scan_date_string(src, weekday, &day, month, &year,
 					&hh, &mm, &ss, zone) < 0) {
@@ -933,14 +1169,6 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 		return 0;
 	}
 
-	/* Y2K compliant :) */
-	if (year < 1000) {
-		if (year < 50)
-			year += 2000;
-		else
-			year += 1900;
-	}
-
 	month[3] = '\0';
 	for (p = monthstr; *p != '\0'; p += 3) {
 		if (!g_ascii_strncasecmp(p, month, 3)) {
@@ -949,6 +1177,37 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 		}
 	}
 
+#ifdef G_OS_WIN32
+	GTimeZone *tz;
+	GDateTime *dt, *dt2;
+
+#if GLIB_CHECK_VERSION(2,68,0)
+	tz = g_time_zone_new_identifier(zone);
+	if (tz == NULL)
+		tz = g_time_zone_new_utc();
+#else
+	tz = g_time_zone_new(zone); // can't return NULL no need to check for it
+#endif
+	dt = g_date_time_new(tz, 1, 1, 1, 0, 0, 0);
+	g_time_zone_unref(tz);
+	dt2 = g_date_time_add_full(dt, year-1, dmonth-1, day-1, hh, mm, ss);
+	g_date_time_unref(dt);
+
+	timer = g_date_time_to_unix(dt2);
+	g_date_time_unref(dt2);
+
+#else
+	struct tm t;
+	time_t tz_offset;
+
+	/* Y2K compliant :) */
+	if (year < 1000) {
+		if (year < 50)
+			year += 2000;
+		else
+			year += 1900;
+	}
+
 	t.tm_sec = ss;
 	t.tm_min = mm;
 	t.tm_hour = hh;
@@ -964,6 +1223,7 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	if (tz_offset != -1)
 		timer += tzoffset_sec(&timer) - tz_offset;
 
+#endif
 	if (dest)
 		procheader_date_get_localtime(dest, len, timer);
 
@@ -1003,34 +1263,44 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 
 /* Added by Mel Hadasht on 27 Aug 2001 */
 /* Get a header from msginfo */
-gint procheader_get_header_from_msginfo(MsgInfo *msginfo, gchar *buf, gint len, gchar *header)
+gint procheader_get_header_from_msginfo(MsgInfo *msginfo, gchar **buf, gchar *header)
 {
 	gchar *file;
 	FILE *fp;
 	HeaderEntry hentry[]={ { NULL, NULL, TRUE  },
-                               { NULL, NULL, FALSE } };
+						   { NULL, NULL, FALSE } };
 	gint val;
 
-        hentry[0].name = header;
-       
 	cm_return_val_if_fail(msginfo != NULL, -1);
+	cm_return_val_if_fail(buf != NULL, -1);
+	cm_return_val_if_fail(header != NULL, -1);
+
+	hentry[0].name = header;
+
 	file = procmsg_get_message_file_path(msginfo);
-	if ((fp = g_fopen(file, "rb")) == NULL) {
-               FILE_OP_ERROR(file, "fopen");
-               g_free(file);
-               return -1;
+	if ((fp = claws_fopen(file, "rb")) == NULL) {
+		FILE_OP_ERROR(file, "claws_fopen");
+		g_free(file);
+		g_free(*buf);
+		*buf = NULL;
+		return -1;
 	}
-	val = procheader_get_one_field(buf,len, fp, hentry);
-	if (fclose(fp) == EOF) {
-		FILE_OP_ERROR(file, "fclose");
+	val = procheader_get_one_field(buf, fp, hentry);
+
+	if (claws_fclose(fp) == EOF) {
+		FILE_OP_ERROR(file, "claws_fclose");
 		claws_unlink(file);
 		g_free(file);
+		g_free(*buf);
+		*buf = NULL;
 		return -1;
 	}
 
 	g_free(file);
-        if (val == -1)
+	if (val == -1) {
+		/* *buf is already NULL in that case, see procheader_get_one_field() */
 		return -1;
+	}
 
 	return 0;
 }
@@ -1082,3 +1352,31 @@ void procheader_entries_free (HeaderEntry *entries)
 	}
 }
 
+gboolean procheader_header_is_internal(const gchar *hdr_name)
+{
+	const gchar *internal_hdrs[] = {
+		"AF:", "NF:", "PS:", "SRH:", "SFN:", "DSR:", "MID:", "CFG:",
+		"PT:", "S:", "RQ:", "SSV:", "NSV:", "SSH:", "R:", "MAID:",
+		"SCF:", "RMID:", "FMID:", "NAID:",
+		"X-Claws-Account-Id:",
+		"X-Claws-Sign:",
+		"X-Claws-Encrypt:",
+		"X-Claws-Privacy-System:",
+		"X-Claws-Auto-Wrapping:",
+		"X-Claws-Auto-Indent:",
+		"X-Claws-End-Special-Headers:",
+		"X-Sylpheed-Account-Id:",
+		"X-Sylpheed-Sign:",
+		"X-Sylpheed-Encrypt:",
+		"X-Sylpheed-Privacy-System:",
+		"X-Sylpheed-End-Special-Headers:",
+	         NULL
+	};
+	int i;
+
+	for (i = 0; internal_hdrs[i]; i++) {
+	        if (!strcmp(hdr_name, internal_hdrs[i]))
+	                return TRUE;
+	}
+	return FALSE;
+}
